@@ -38,7 +38,14 @@
 #include "ar-string-utils.h"
 #include "ar-gsettings.h"
 
+#ifdef HAVE_CLUTTER
+#include "ar-clutter-embed.h"
+#include "ar-style.h"
+#include "baize.h"
+#include "board.h"
+#else
 #include "board-noclutter.h"
+#endif
 
 #include "ar-card-theme.h"
 #include "ar-card-themes.h"
@@ -89,7 +96,13 @@ struct _AisleriotWindowPrivate
 {
   AisleriotGame *game;
   ArStyle *board_style;
+#ifdef HAVE_CLUTTER
+  ArClutterEmbed *board;
+  ClutterActor *baize_actor;
+  ClutterActor *board_actor;
+#else
   AisleriotBoard *board;
+#endif
 
   ArCardThemes *theme_manager;
   ArCardTheme *theme;
@@ -319,7 +332,11 @@ undo_cb (GtkAction *action,
   AisleriotWindowPrivate *priv = window->priv;
 
   /* If a move is in progress, cancel it before changing the game! */
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
   aisleriot_board_abort_move (priv->board);
+#endif
 
   aisleriot_game_undo_move (priv->game);
 }
@@ -330,7 +347,11 @@ redo_cb (GtkAction *action,
 {
   AisleriotWindowPrivate *priv = window->priv;
 
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
   aisleriot_board_abort_move (priv->board);
+#endif
 
   aisleriot_game_redo_move (priv->game);
 }
@@ -935,6 +956,24 @@ sound_toggle_cb (GtkToggleAction *action,
 
 #endif /* ENABLE_SOUND */
 
+#ifdef HAVE_CLUTTER
+
+static void
+animations_toggle_cb (GtkToggleAction *action,
+                      AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  gboolean enabled;
+
+  enabled = gtk_toggle_action_get_active (action);
+
+  ar_style_set_enable_animations (priv->board_style, enabled);
+  
+  ar_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), enabled);
+}
+
+#endif /* HAVE_CLUTTER */
+
 static void
 show_hint_cb (GtkAction *action,
               AisleriotWindow *window)
@@ -1732,7 +1771,7 @@ game_exception_cb (AisleriotGame *game,
   gtk_widget_show (dialog);
 }
 
-#if defined(ENABLE_SOUND)
+#if defined(HAVE_CLUTTER) || defined(ENABLE_SOUND)
 
 static void
 settings_changed_cb (GtkSettings *settings,
@@ -1749,12 +1788,23 @@ settings_changed_cb (GtkSettings *settings,
   else
     name = NULL;
 
+#ifdef HAVE_CLUTTER
+  if (name == NULL || strcmp (name, "gtk-enable-animations") == 0) {
+    g_object_get (settings, "gtk-enable-animations", &enabled, NULL);
+
+    action = gtk_action_group_get_action (priv->action_group, "Animations");
+    gtk_action_set_visible (action, enabled);
+  }
+#endif /* HAVE_CLUTTER */
+
+#if defined(ENABLE_SOUND)
   if (name == NULL || strcmp (name, "gtk-enable-event-sounds") == 0) {
     g_object_get (settings, "gtk-enable-event-sounds", &enabled, NULL);
 
     action = gtk_action_group_get_action (priv->action_group, "Sound");
     gtk_action_set_visible (action, enabled);
   }
+#endif /* ENABLE_SOUND */
 }
 
 static void
@@ -1782,11 +1832,17 @@ screen_changed_cb (GtkWidget *widget,
 
   settings = gtk_widget_get_settings (widget);
   settings_changed_cb (settings, NULL, window);
+#ifdef HAVE_CLUTTER
+  g_signal_connect (settings, "notify::gtk-enable-animations",
+                    G_CALLBACK (settings_changed_cb), window);
+#endif
+#if defined (ENABLE_SOUND)
   g_signal_connect (settings, "notify::gtk-enable-event-sounds",
                     G_CALLBACK (settings_changed_cb), window);
+#endif
 }
 
-#endif /* ENABLE_SOUND */
+#endif /* HAVE_CLUTTER || ENABLE_SOUND */
 
 /*
  * aisleriot_window_set_freecell_mode:
@@ -1832,6 +1888,35 @@ board_status_message_cb (AisleriotBoard *board,
   }
 }
 
+#ifdef HAVE_CLUTTER
+
+static void
+board_cursor_cb (AisleriotBoard *board,
+                 int cursor_type,
+                 ArClutterEmbed *embed)
+{
+  ar_clutter_embed_set_cursor (embed, (ArCursorType) cursor_type);
+}
+
+static void
+board_error_bell_cb (AisleriotBoard *board,
+                     ArClutterEmbed *embed)
+{
+  gtk_widget_error_bell (GTK_WIDGET (embed));
+}
+
+static void
+embed_size_allocate_cb (ArClutterEmbed *embed,
+                        GtkAllocation *allocation,
+                        AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  clutter_actor_set_size (priv->board_actor, allocation->width, allocation->height);
+}
+
+#endif /* HAVE_CLUTTER */
+
 /* Class implementation */
 
 G_DEFINE_TYPE (AisleriotWindow, aisleriot_window, GTK_TYPE_WINDOW)
@@ -2010,6 +2095,12 @@ aisleriot_window_init (AisleriotWindow *window)
       G_CALLBACK (sound_toggle_cb),
       FALSE /* not active by default */ },
 #endif /* ENABLE_SOUND */
+#ifdef HAVE_CLUTTER
+   { "Animations", NULL, N_("_Animations"), NULL,
+      N_("Whether or not to animate card moves"),
+      G_CALLBACK (animations_toggle_cb),
+      FALSE /* not active by default */ },
+#endif /* HAVE_CLUTTER */
   };
 
   static const char names[][16] = {
@@ -2056,6 +2147,9 @@ aisleriot_window_init (AisleriotWindow *window)
           "<menuitem action='ClickToMove'/>"
 #ifdef ENABLE_SOUND
           "<menuitem action='Sound'/>"
+#endif
+#ifdef HAVE_CLUTTER
+          "<menuitem action='Animations'/>"
 #endif
         "</menu>"
         "<menu action='OptionsMenu'/>"
@@ -2112,6 +2206,9 @@ aisleriot_window_init (AisleriotWindow *window)
   guint i;
   GtkStatusbar *statusbar;
   GtkWidget *statusbar_hbox, *label, *time_box;
+#ifdef HAVE_CLUTTER
+  ClutterContainer *stage;
+#endif
 
   g_assert (G_N_ELEMENTS (names) == LAST_ACTION);
 
@@ -2125,7 +2222,34 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->board_style = ar_style_new ();
 
+#ifdef HAVE_CLUTTER
+  priv->board = ar_clutter_embed_new (priv->board_style);
+
+  priv->baize_actor = aisleriot_baize_new ();
+
+  stage = CLUTTER_CONTAINER (gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (priv->board)));
+  clutter_container_add (stage, priv->baize_actor, NULL);
+  /* FIXMEchpe: how to ensure this is ALWAYS the lowest actor? */
+  clutter_actor_lower_bottom (priv->baize_actor);
+
+  priv->board_actor = aisleriot_board_new (priv->board_style, priv->game);
+  clutter_container_add (stage, priv->board_actor, NULL);
+
+  /* FIXMEchpe */
+  clutter_stage_set_key_focus (CLUTTER_STAGE (stage), priv->board_actor);
+
+  g_signal_connect_after (priv->board, "size-allocate",
+                          G_CALLBACK (embed_size_allocate_cb), window);
+
+  g_signal_connect (priv->board_actor, "request-cursor",
+                    G_CALLBACK (board_cursor_cb), priv->board);
+  g_signal_connect (priv->board_actor, "error-bell",
+                    G_CALLBACK (board_error_bell_cb), priv->board);
+
+  /* FIXMEchpe: unref baize & board_actor here? */
+#else
   priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->board_style, priv->game));
+#endif /* HAVE_CLUTTER */
 
   theme_name = ar_conf_get_string (NULL, aisleriot_conf_get_key (CONF_THEME), NULL);
   theme = ar_card_themes_get_theme_by_name (priv->theme_manager, theme_name);
@@ -2176,8 +2300,13 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->game_message_id = gtk_statusbar_get_context_id (priv->statusbar, "board-message");
 
+#ifdef HAVE_CLUTTER
+  g_signal_connect (priv->board_actor, "status-message",
+                    G_CALLBACK (board_status_message_cb), window);
+#else
   g_signal_connect (priv->board, "status-message",
                     G_CALLBACK (board_status_message_cb), window);
+#endif
 
   gtk_window_set_has_resize_grip (GTK_WINDOW (window), TRUE);
 
@@ -2270,12 +2399,19 @@ aisleriot_window_init (AisleriotWindow *window)
 
   set_fullscreen_actions (window, FALSE);
 
-#if defined(ENABLE_SOUND)
+#ifdef HAVE_CLUTTER
+  action = gtk_action_group_get_action (priv->action_group, "Animations");
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+                                ar_conf_get_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), NULL));
+
+#endif /* HAVE_CLUTTER */
+
+#if defined(HAVE_CLUTTER) || defined(ENABLE_SOUND)
   /* Set the action visibility and listen for animation and sound mode changes */
   screen_changed_cb (GTK_WIDGET (window), NULL, window);
   g_signal_connect (window, "screen-changed",
                     G_CALLBACK (screen_changed_cb), window);
-#endif /* ENABLE_SOUND */
+#endif /* HAVE_CLUTTER || ENABLE_SOUND */
 
   /* Now set up the widgets */
   main_vbox = gtk_vbox_new (FALSE, 0);
@@ -2331,12 +2467,12 @@ aisleriot_window_dispose (GObject *object)
 {
   AisleriotWindow *window = AISLERIOT_WINDOW (object);
   AisleriotWindowPrivate *priv = window->priv;
-
-#ifdef ENABLE_SOUND
+  
+#ifdef HAVE_CLUTTER
   g_signal_handlers_disconnect_by_func (gtk_widget_get_settings (GTK_WIDGET (window)),
                                         G_CALLBACK (settings_changed_cb),
                                         window);
-#endif /* ENABLE_SOUND */
+#endif /* HAVE_CLUTTER */
 
   if (priv->hint_dialog) {
     gtk_widget_destroy (priv->hint_dialog);
@@ -2376,6 +2512,10 @@ aisleriot_window_finalize (GObject *object)
   AisleriotWindow *window = AISLERIOT_WINDOW (object);
   AisleriotWindowPrivate *priv = window->priv;
 
+#ifdef HAVE_CLUTTER
+  g_object_unref (priv->board_style);
+#endif /* HAVE_CLUTTER */
+
   if (priv->theme) {
     g_object_unref (priv->theme);
   }
